/*
  gDisplay_volt_time2.c
    October 11, 2015
   * Menu that uses a COG.
   * Displays Menu, a volt, clock, auto running in a COG.
   * 
*/
#include "simpletools.h"
#include "badgetools.h"
#include "datetime.h"

/* Datetime */
datetime dts = {2000, 1, 1, 0, 0, 0};  // datetime seed value.

datetime dta, dtt;
/****************/

/* Voltage gauge*/
float calcVt(float batteryVolts, int decayMicros);
float batVolts(float vt);

float RC = 360000.0 * (1.0 / 100000000.0);
float vt, vbat;
/****************/

/* Case statement variables */
int states;
volatile int myLock;

/* Function declarations */
void NewClear();  // Clears screen lines 0 - 6.
void bLine();  // Clean a line.

/* COG */
void System_base();
volatile int OnOff;   // For IR signal adjust.
/******************/

int main()
{
  // Add startup code here.
  badge_setup();
  dt_start(&dts);  // Start the clock  
  myLock=locknew();
  lockclr(myLock);
/* Voltage RCT value. */
  vt = calcVt(4.17, 1453);  // Universal RCT value.
/******************/
  OnOff = 1;  // IR signal seek is OFF.
  clear();
/* Setup text default font size.*/
// To be used in the COG, at startup.  
  text_size(SMALL);

/* Display IR status */  
  if(OnOff == 0)
  {
    text_size(LARGE);
    oledprint("IR ON");
    pause(2000);
  }
  else
  {
    text_size(LARGE);
    oledprint("IR OFF");
    pause(2000);    
  }
  text_size(SMALL);
/******************/
  
/* Start the system COG. */   
  cog_run(System_base,128);        
 
  while(1)
  {
    // Add main loop code here.
    /* Using continue in the switch statement
       brings you back to this point. */
    while(lockset(myLock));  // Lockit
    NewClear();  // Clear 0 - 6 and set cursor to (0,0)
    text_size(SMALL);
    cursor(0,0);
    oledprint("Any BTN Menu    ");
    lockclr(myLock);  // Release it.
         
    while(!buttons());  // No button press.
    /* Menu setup. */
    while(lockset(myLock));  // Lockit
    NewClear();
    oledprint("P17           ");
    cursor(0,1);
    oledprint("           P27");
    cursor(0,2);
    oledprint("P16           ");
    cursor(0,3);
    oledprint("           P26");
    cursor(0,4);
    oledprint("P15-IR signal ");
    cursor(0,5);
    oledprint("      Date-P25");
    cursor(0,6);
    oledprint("      OSH     ");
    lockclr(myLock);  // Release it.
        
    while((states = buttons()) == 0);  // Wait for a button press.
    switch(states)
    {
      case 0b0100000:  //p17
        while(lockset(myLock));  // Lockit
        cursor(0,6);   // Use temp line.
        oledprint("B17 ");
        pause(2000);
        lockclr(myLock);  // Release it.
        continue;    // Goes to the while(1)
      /*********************/
      case 0b0000001:  //P27
        while(lockset(myLock));  // Lockit
        cursor(0,6);   // Use temp line.
        oledprint("B27 ");
        pause(2000);
        lockclr(myLock);  // Release it.
        continue;    // Goes to the while(1)
      /*********************/
      case 0b0010000:  //P16
        while(lockset(myLock));  // Lockit
        cursor(0,6);  // Use temp line.
        oledprint("B16 ");
        pause(2000);
        lockclr(myLock);  // Release it.     
        continue;    // Goes to the while(1)
      /*********************/
      case 0b0000010:  //P26
        while(lockset(myLock));  // Lockit
        cursor(0,6);   // Use temp line.
        oledprint("B26 ");
        pause(2000);
        lockclr(myLock);  // Release it.     
        continue;    // Goes to the while(1)
      /*********************/
      case 0b0001000:  //P15
        while(lockset(myLock));  // Lockit
        NewClear();
        text_size(SMALL);
        oledprint("P17Yes    NoP27");
        while((states = buttons()) == 0); // Wait for button
        while(1)
        {
          if(states == 0b0100000)  // Yes, IR signal ON.
          {
            OnOff = 0;  // Turns IR signal ON
            NewClear();
            text_size(LARGE);
            oledprint("IR ON");
            pause(2000);
            text_size(SMALL);
            break;
          }
          if(states == 0b0000001);
          {
            OnOff = 1;  // Turns IR signal OFF
            NewClear();
            text_size(LARGE);
            oledprint("IR OFF");
            pause(2000);
            text_size(SMALL);
            break;
          }
        }          
        lockclr(myLock);  // Release it.
        continue;    // Goes to the while(1)
      /*********************/
      case 0b0000100:  //P25 Date
        while(lockset(myLock));  // Lockit
        NewClear();
        text_size(LARGE);
        oledprint("%02d/%02d/%02d",dts.mo, dts.d, dts.y % 100);
        pause(2000);
        text_size(SMALL);
        lockclr(myLock);  // Release it.
        continue;    // Goes to the while(1)
      /*********************/
      case 0b1000000:  //OSH
        while(lockset(myLock));  // Lockit
        cursor(0,6);   // Use status line.
        oledprint("bOSH");
        pause(2000);
        lockclr(myLock);  // Release it.
        continue;    // Goes to the while(1)        
      /*********************/
      default:
        continue;    // Goes to the while(1)
    }          
  }  
}


/* COG for running system base utilty. */
void System_base()
{
  char up_time[] = "T";    // For time update.
  char ack_ir[] = "TO\n";  // Send ack.
  
  while(1)
  {
/* Show voltage */
    /* Lockit */
    while(lockset(myLock));
    cursor(9,7);
    // Voltage
    vbat = batVolts(vt);
    pause(100);
    oledprint("%1.2f V", vbat);
    lockclr(myLock);
    /* Release it. */
/*********************/

/* Auto update of clock */
    /* Lockit */
    while(lockset(myLock));
    cursor(0,7);
    text_size(SMALL);
    oledprint("%02d:%02d:%02d ",dts.h, dts.m, dts.s);
    lockclr(myLock);
    /* Release it. */
    if(OnOff == 1)  // IR signal seek is off.
    {
      continue;
    }
    else
    {
      irclear();  
      if(irscan("%s",up_time))  // Check for ir signal.
      {
        rgb(R,GREEN);  // Turn on, starting up.
        irprint("%s",ack_ir);  // Send ack, OK, start update.
        irclear();  // Clear IR buffer.
       memset(&dtt, 0, sizeof(datetime));
          while(1)
          {  // Waiting for data         
            irscan("%d%d%d%d%d%d",
            &dtt.y, &dtt.mo, &dtt.d, &dtt.h, &dtt.m, &dtt.s);
            if(dtt.y > 0)
            {
              dt_copy(&dts, &dtt); //copy new values
              break;  // Break out of this while(1).
            }
          }
      rgb(R,OFF);  // Turn off, update complete.
      irprint("%s",ack_ir);  // Send all done  ack.
      }
    }  // if OnOff
        
/*********************/
  }      
}


float calcVt(float voltsBattery, int microsDecay)
{
  float voltsThreshold, tDecay;
  tDecay = ((float) microsDecay) / 1000000.0;
  voltsThreshold = voltsBattery * (1 - exp(-tDecay / RC));
  return voltsThreshold;
}
  
float batVolts(float voltsThreshold)
{
  float voltsBattery, microsDecay;
  low(0);
  pause(1);
  microsDecay = ((float) rc_time(0, 0)) / 1000000.0;
  voltsBattery = voltsThreshold / (1 - exp(-microsDecay / RC));
  return voltsBattery;
}

/* To be used for clearing lines 0 - 6 */
void NewClear()
{
  cursor(0,0);
  bLine();
  cursor(0,1);
  bLine();
  cursor(0,2);
  bLine();
  cursor(0,3);
  bLine();
  cursor(0,4);
  bLine();
  cursor(0,5);
  bLine();
  cursor(0,6);
  bLine();
  cursor(0,0);
}


/* Blanks a line. */
void bLine()
{
  oledprint("                ");
}    
